# Contributing

Contributions are welcome, and they are greatly appreciated. Every
little bit helps, and credit will always be given.

You can contribute in many ways:

## Types of Contributions

### Report Bugs

Report bugs at https://github.com/tomtom-international/vault-anyconfig/issues.

If you are reporting a bug, please include:

* Your operating system name and version.
* Any details about your local setup that might be helpful in troubleshooting.
* Detailed steps to reproduce the bug.

### Fix Bugs

Look through the GitHub issues for bugs. Anything tagged with "bug" and
"help wanted" is open to whoever wants to implement it.

### Implement Features

Look through the GitHub issues for features. Anything tagged with
"enhancement" and "help wanted" is open to whoever wants to implement
it.

### Write Documentation

vault-anyconfig could always use more documentation,
whether as part of the official vault-anyconfig docs,
in docstrings, or even on the web in blog posts, articles, and such.

### Submit Feedback

The best way to send feedback is to file an issue at
https://github.com/tomtom-international/vault-anyconfig/issues.

If you are proposing a feature:

* Explain in detail how it would work.
* Keep the scope as narrow as possible, to make it easier to implement.
* Remember that this is a volunteer-driven project, and that contributions are welcome :)

## Get Started

Ready to contribute? Here's how to set up **vault-anyconfig** for local development.

1. Fork the **vault-anyconfig** repo on Github.

2. Clone your fork locally:

        $ git clone git@github.com:your_name_here/vault-anyconfig.git

3. Install your local copy into a virtualenv. Assuming you have virtualenv installed,
   this is how you set up your fork for local development:

        $ cd vault-anyconfig/
        $ virtualenv env
        $ source env/bin/activate
        $ python setup.py develop

4. Create a branch for local development:

        $ git checkout -b name-of-your-bugfix-or-feature

  Now you can make your changes locally.

5. When you're done making changes, check that your changes pass pylint
   and the tests, including testing other Python versions with tox:

        $ tox

  To get tox, just pip install it into your virtualenv.

6. Commit your changes and push your branch to Github:

        $ git add .
        $ git commit -m "Your detailed description of your changes."
        $ git push origin name-of-your-bugfix-or-feature

7. Submit a pull request through the Github website.

## Pull Request Guidelines

Before you submit a pull request, check that it meets these guidelines:

1. The pull request should include tests.
2. If the pull request adds functionality, the docs should be updated.
   Put your new functionality into a function with a docstring, and add the feature to the list in README.md.
3. The pull request should work for Python 3.5, 3.6 and 3.7, and for PyPy. Check https://dev.azure.com/tomtomweb/GitHub-TomTom-International/_build and make sure that the tests pass for all supported Python versions.

## Tips

To run a subset of tests:

    $ py.test tests/test_vault_anyconfig.py

## Deploying

Deployment can only be done by the project maintainers and is done on-demand via the Azure CI.

Select the [vault-anyconfig build pipeline](https://dev.azure.com/tomtomweb/GitHub-TomTom-International/_build) and click
on the **Queue** button and run a build with the following settings:

* Branch: `master`
* Commit: leave it empty
* Variables:
  * `release: true`

